from AccessControl.SecurityManagement import newSecurityManager
from castle.cms._scripts import mjson
from castle.cms._scripts.importtypes import get_import_type
from lxml.html import fromstring
from lxml.html import tostring
from OFS.interfaces import IFolder
from plone import api
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.app.textfield.value import RichTextValue
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from zope.component import getUtility
from zope.component.hooks import setSite

import argparse
import logging
import os
import transaction
import pdb

logger = logging.getLogger('castle.cms')

# ToDo:
#   - do NOT export images
#       - only images that are referenced in content
#   - lookup reference, if image only used once, add as lead image
#   - other images, if not referenced elsewhere, include inline
#       as data uris
#   - links that go to files should be moved to s3 and be converted
#       into file objects
#   - if default page and only page inside folder, replace folder
#


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default='Castle')
parser.add_argument(
    '--export-directory', dest='export_directory', default='export')
parser.add_argument('--import-paths', dest='import_paths', default=False)
parser.add_argument('--skip-paths', dest='skip_paths', default=False)

parser.add_argument('--overwrite', dest='overwrite', default=False)
parser.add_argument('--admin-user', dest='admin_user', default='admin')
parser.add_argument('--ignore-uuids', dest='ignore_uuids', default=False)
parser.add_argument(
    '--stop-if-exception', dest='stop_if_exception', default=False)
parser.add_argument(
    '--pdb-if-exception', dest='pdb_if_exception', default=False)
parser.add_argument('--skip-existing', dest='skip_existing', default=True)
parser.add_argument(
    '--skip-transitioning', dest='skip_transitioning', default=False)
parser.add_argument(
    '--skip-types', dest='skip_types', default=','.join([
        "collective.cover.content",
        "FormFolder",
        "FormMailerAdapter",
        "FormTextField",
        "FormStringField",
        "FormThanksPage",
        "FormSaveDataAdapter",
        "FormSaveData2ContentAdapter",
        "FormSelectionField",
        "Topic"]))
parser.add_argument('--only-types', dest='only_types', default=False)

# Put files/videos/etc in to the repository paths by default,
# Set true to retain exported path
parser.add_argument('--retain-paths', dest='retain_paths', default=False)

args, _ = parser.parse_known_args()

ignore_uuids = args.ignore_uuids
stop_if_exception = args.stop_if_exception
pdb_if_exception = args.pdb_if_exception
retain_paths = args.retain_paths

if args.import_paths:
    import_paths = args.import_paths.split(',')
else:
    import_paths = False
if args.skip_paths:
    skip_paths = args.skip_paths.split(',')
else:
    skip_paths = False


if args.only_types:
    only_types = args.only_types.split(',')
else:
    only_types = False
if args.skip_types:
    skip_types = args.skip_types.split(',')
else:
    skip_types = False

user = app.acl_users.getUser(args.admin_user)  # noqa
try:
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa
except Exception:
    logger.error('Unknown admin user; '
                 'specify an existing Zope admin user with --admin-user '
                 '(default is admin)')
    exit(-1)
site = app[args.site_id]  # noqa
setSite(site)


def traverse(path):
    folder = site  # noqa
    for part in path.strip('/').split('/'):
        if part in folder.objectIds():
            folder = folder[part]
        else:
            return


def relpath(obj):
    return '/'.join(obj.getPhysicalPath())[len(
        '/'.join(site.getPhysicalPath())) + 1:]


def recursive_create_path(path):
    if path in ('/', ''):
        folder = site
    else:
        path = path.lstrip('/')
        folder = traverse(path)
        if folder is not None and not IFolder.providedBy(folder):
            api.content.delete(folder)
            folder = None
        if folder is None:
            # Need to create folders up to where we want content
            # we'll walk it up create folders as needed
            folder = site
            for part in path.split('/'):
                try:
                    ob = folder[part]
                    if not IFolder.providedBy(ob):
                        logger.warn('Existing object in traversal path is not folderish. Removing it.')
                        api.content.delete(ob)
                        raise KeyError()
                    else:
                        folder = ob
                except (KeyError, AttributeError):
                    fpath = os.path.join(
                        args.export_directory,
                        relpath(folder),
                        part,
                        '__folder__')
                    imported = False
                    if os.path.exists(fpath):
                        imported = import_object(fpath)
                    if not imported:
                        logger.info("Creating plain Folder (no __folder__ file found), %s" % part)
                        folder = api.content.create(
                            type='Folder',
                            id=part,
                            title=part.capitalize(),
                            container=folder)
                        bdata = ILayoutAware(folder)
                        bdata.contentLayout = '++contentlayout++castle/folder-query.html'
    return folder


_importable_fields = (
    'title',
    'description',
)


_read_phase = {}


def read_object(filepath):
    # disable, we don't use
    pass


def fix_html_images(obj):
    try:
        html = obj.text.raw
    except Exception:
        return
    if not html:
        return
    changes = False
    try:
        dom = fromstring(html)
    except Exception:
        return
    for el in dom.cssselect('img'):
        src = el.attrib.get('src', '')
        if 'resolveuid' not in src:
            continue
        if '@@images' in src:
            continue
        parts = src.split('/')
        uid = parts[1]
        scale = '/'.join(parts[2:])
        if scale:
            scale = scale.replace('image_', '')
        src = 'resolveuid/%s/@@images/image' % uid
        if scale:
            src += '/' + scale
        attribs = {
            'src': src,
            'data-linktype': 'image',
            'data-val': uid
        }
        if scale:
            attribs['data-scale'] = scale
        el.attrib.update(attribs)
        changes = True
    if changes:
        obj.text = RichTextValue(
            tostring(dom), mimeType=obj.text.mimeType,
            outputMimeType=obj.text.outputMimeType)


def import_object(filepath):
    fi = open(filepath)
    file_read = fi.read()
    fi.close()
    try:
        data = mjson.loads(file_read)
    except Exception:
        print("Skipping {}; Unable to read JSON data".format(filepath))
        return
    if filepath.endswith('__folder__'):
        filepath = '/'.join(filepath.split('/')[:-1])

    skipped = False
    if data['portal_type'] in skip_types:
        print('Skipping omitted type {type}'
                                        .format(type=data['portal_type']))
        skipped = True
    if only_types and data['portal_type'] not in only_types:
        print("Skipping {type} at {path}, not in only_types."
                            .format(type=data['portal_type'], path=filepath))
        skipped = True
    if import_paths:
        do_import = False
        for import_path in import_paths:
            if filepath.startswith('{}/{}'.format(args.export_directory, import_path)):
                do_import = True
            if import_path.startswith(filepath[len(args.export_directory):].lstrip('/') + '/'):
                # Don't skip folders on the way to import_paths
                do_import = True
        if not do_import:
            print("Skipping {path}, not in import_paths"
                                .format(path=filepath))
            skipped = True
    if skip_paths:
        for skip_path in skip_paths:
            if filepath.lower().startswith('{}/{}'.format(args.export_directory, skip_path)):
                print("Skipping {path}, in skip_paths"
                                .format(path=filepath))
                skipped = True
    if skipped:
        if os.path.isdir(filepath) and len(os.listdir(filepath)):
            logger.warn('{path} contains additional content that will be '
                        'skipped.'.format(path=filepath))
        return

    original_path = filepath[len(args.export_directory):]
    if retain_paths:
        importtype = get_import_type(data, original_path, 'retain_paths')
    else:
        importtype = get_import_type(data, original_path)
    path = importtype.get_path()

    folder = recursive_create_path('/'.join(path.split('/')[:-1]))
    if folder is None:
        logger.warn('Skipped {} because of creation error'.format(filepath))
        return
    _id = path.split('/')[-1]

    create = True
    if _id in folder.objectIds():
        if args.overwrite:
            existing = folder[_id]
            if IFolder.providedBy(existing):
                if len(existing.objectIds()):
                    print("OVERWRITE: Deleting non-empty folder {path}".format(path=path))
            else:
                print("OVERWRITE: Deleting content item at {path}".format(path=path))
            api.content.delete(folder[_id])
        else:
            create = False

    creation_data = importtype.get_data()
    creation_data['container'] = folder

    aspect = ISelectableConstrainTypes(folder, None)
    if aspect:
        if (aspect.getConstrainTypesMode() != 1 or
                [creation_data['type']] != aspect.getImmediatelyAddableTypes()):
            aspect.setConstrainTypesMode(1)
            aspect.setImmediatelyAddableTypes([creation_data['type']])

    if create:
        if ignore_uuids and '_plone.uuid' in creation_data:
            del creation_data['_plone.uuid']

        obj = None
        if not args.overwrite and (_id in folder.objectIds()):
            print('Skipping {path}, already exists. Use --overwrite to'
                                        ' create anyway.'.format(path=path))
            return
        elif (not ignore_uuids and api.content.get(UID=creation_data['_plone.uuid']) is not None):
            logger.warn('Skipping {path}, content with its UUID already exists.'
                                        'Use --ignore-uuids to create anyway.'
                                                            .format(path=path))
            return
        else:
            try:
                obj = api.content.create(safe_id=True, **creation_data)
                print('Created {path}'.format(path=path))
            except api.exc.InvalidParameterError:
                if stop_if_exception:
                    logger.error('Error creating content {}'.format(filepath), exc_info=True)
                    if pdb_if_exception:
                        pdb.set_trace()
                    raise
                logger.error('Error creating content {}'
                                            .format(filepath), exc_info=True)
                return


# TODO check default folder pages came over as folder with rich text tile
# TODO any folder pages without default page should have content listing tile
    else:
        obj = folder[_id]
        for key, value in creation_data.items():
            if key not in ('id', 'type'):
                setattr(obj, key, value)

    if obj is not None:
        if path != original_path:
            storage = getUtility(IRedirectionStorage)
            rpath = os.path.join('/'.join(site.getPhysicalPath()),
                                 original_path.strip('/'))
            storage.add(rpath, "/".join(obj.getPhysicalPath()))

        obj.contentLayout = importtype.layout
        importtype.post_creation(obj)
        if not args.skip_transitioning and data['state']:
            try:
                print('Transitioning %s to %s' % (obj.id, data['state']))
                api.content.transition(obj, to_state=data['state'])
            except Exception:
                logger.error("Error transitioning %s to %s, maybe workflows"
                                "don't match up" % (obj.id, data['state']))
                # pass
                if stop_if_exception:
                    if pdb_if_exception:
                        pdb.set_trace()
                    raise

        # set workflow / review history
        if 'review_history' in data:
            review_history = data['review_history']
            wtool = api.portal.get_tool(name='portal_workflow')
            chain = wtool.getChainFor(obj)
            if len(chain):
                workflow_id = chain[0]
                for h in review_history:
                    wtool.setStatusOf(workflow_id, obj, h)
                if len(chain) != 1:
                    return logger.warn('There should be 1 workflow for %s but'
                                        'there are %i' % (path, len(chain)))
        else:
            return logger.warn('No review history on {obj}'.format(obj=obj))

        fix_html_images(obj)
        obj.reindexObject()
        return True


def import_content(path, count=0):
    this_folder = os.path.join(path, '__folder__')
    if os.path.exists(this_folder):
        imported = import_object(this_folder)
        if imported:
            count += 1
        else:
            # folder object exists in export,but errored or was skipped
            return count
    for filename in os.listdir(path):
        if filename in ('.DS_Store', '__folder__'):
            continue
        filepath = os.path.join(path, filename)
        if os.path.isdir(filepath):
            count = import_content(filepath, count)
        else:
            try:
                imported = import_object(filepath)
                if imported:
                    count += 1
            except Exception:
                logger.error('Error importing {path}'.format(path=filepath),
                                                                exc_info=True)
                if stop_if_exception:
                    if pdb_if_exception:
                        pdb.set_trace()
                    raise
        if count and count % 50 == 0:
            print('%i processed, committing' % count)
            transaction.commit()
            app._p_jar.invalidateCache()  # noqa
            transaction.begin()
            app._p_jar.sync()  # noqa
    if path == args.export_directory:
        transaction.commit()
        app._p_jar.invalidateCache()  # noqa
    return count


def read_pass(path):
    for filename in sorted(os.listdir(path)):
        if filename in ('.DS_Store', '__folder__'):
            continue
        filepath = os.path.join(path, filename)
        if os.path.isdir(filepath):
            read_pass(filepath)
        else:
            read_object(filepath)


if __name__ == '__main__':
    # logger.info('Doing read pass...')
    # read_pass(args.export_directory)

    print('------------------------------')
    print('Start importing')
    print('------------------------------')
    if args.overwrite:
        print('------------------------------')
        print('Importing with overwrite enabled')
        print('------------------------------')
    count = import_content(args.export_directory)
    print('Created {count} Content Items'.format(count=count))
    transaction.commit()



# -*- coding: utf-8 -*-
import json
import unittest
from copy import deepcopy

from castle.cms.patterns import CastleSettingsAdapter
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.upgrades import upgrade_2_2_0
from castle.cms.utils import get_upload_fields
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles
from plone.registry.field import List
from plone.registry.interfaces import IRegistry
from plone.registry.record import Record
from zope.component import getUtility


class TestFileUploadFields(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_fallback_to_default_settings_until_upgrade_done(self):
        registry = getUtility(IRegistry)
        del registry.records['castle.file_upload_fields']
        record = Record(List(), [u'title', u'description'])
        registry.records['castle.required_file_upload_fields'] = record
        fields = get_upload_fields(registry)
        self.assertTrue(fields[0]['required'])
        self.assertTrue(fields[1]['required'])
        self.assertEquals(len(fields), 4)

    def test_get_settings(self):
        registry = getUtility(IRegistry)
        fields = deepcopy(registry['castle.file_upload_fields'])
        for field in fields:
            field['required'] = unicode(field['required']).lower()
        self.assertEquals(len(get_upload_fields(registry)), 4)
        fields.append({
            u'name': u'foobar',
            u'label': u'Foobar',
            u'widget': u'text',
            u'required': u'false',
            u'for-file-types': u'*'
        })
        registry['castle.file_upload_fields'] = fields
        self.assertEquals(len(get_upload_fields(registry)), 5)

    def test_handle_bad_field_values(self):
        # requires fields defined
        registry = getUtility(IRegistry)
        fields = deepcopy(registry['castle.file_upload_fields'])
        fields.append({})
        registry['castle.file_upload_fields'] = deepcopy(fields)
        self.assertEquals(len(get_upload_fields(registry)), 4)

        # auto fill in for missing
        fields.append({
            u'name': u'foobar'
        })
        registry['castle.file_upload_fields'] = fields

        fields = get_upload_fields(registry)
        self.assertEquals(len(fields), 5)
        self.assertEquals(fields[-1]['label'], 'Foobar')
        self.assertEquals(fields[-1]['required'], False)
        self.assertEquals(fields[-1]['widget'], 'text')
        self.assertEquals(fields[-1]['for-file-types'], '*')

    def test_get_pattern_settings(self):
        patterns = CastleSettingsAdapter(self.portal, self.request, None)
        data = patterns()
        self.assertTrue('data-required-file-upload-fields' in data)
        self.assertEquals(data['data-required-file-upload-fields'], json.dumps([u'title']))
        self.assertEquals(
            len(json.loads(data['data-file-upload-fields'])), 4)

    def test_migrate_settings(self):
        registry = getUtility(IRegistry)
        del registry.records['castle.file_upload_fields']
        record = Record(List(), [u'title'])
        registry.records['castle.required_file_upload_fields'] = record

        self.assertIn('castle.required_file_upload_fields', registry.records._fields)
        self.assertIn('castle.required_file_upload_fields', registry.records._values)

        upgrade_2_2_0.upgrade(self.portal)
        self.assertEquals(len(registry['castle.file_upload_fields']), 4)

        self.assertTrue(registry['castle.file_upload_fields'] is not None)
        self.assertTrue(registry.get('castle.required_file_upload_fields') is None)

        # check registry cleaned up as well
        self.assertIn('castle.file_upload_fields', registry.records._fields)
        self.assertIn('castle.file_upload_fields', registry.records._values)
        self.assertNotIn('castle.required_file_upload_fields', registry.records._fields)
        self.assertNotIn('castle.required_file_upload_fields', registry.records._values)

        # also, test we can safely run it again without overwritting existing values
        fields = deepcopy(registry['castle.file_upload_fields'])
        fields.append({
            u'name': u'foobar',
            u'label': u'Foobar',
            u'widget': u'text',
            u'required': u'false',
            u'for-file-types': u'*'
        })
        registry['castle.file_upload_fields'] = fields
        upgrade_2_2_0.upgrade(self.portal)
        self.assertEquals(len(registry['castle.file_upload_fields']), 5)



import json
import time

from castle.cms import _, authentication, cache, texting
from castle.cms.interfaces import (IAuthenticator, ISecureLoginAllowedView,
                                   ISiteSchema)
from castle.cms.pwexpiry.utils import days_since_event
from castle.cms.utils import get_managers, send_email, strings_differ
from DateTime import DateTime
from plone import api
from plone.protect.authenticator import createToken
from plone.protect.interfaces import IDisableCSRFProtection
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.PasswordResetTool.PasswordResetTool import (ExpiredRequestError,
                                                          InvalidRequestError)
from ZODB.POSException import ConnectionStateError
from zope.component import getMultiAdapter, getUtility
from zope.component.interfaces import ComponentLookupError
from zope.i18n import translate
from zope.interface import alsoProvides, implements


class SecureLoginView(BrowserView):
    implements(ISecureLoginAllowedView)

    def __init__(self, context, request):
        super(SecureLoginView, self).__init__(context, request)
        self.auth = self.authenticator = getMultiAdapter(
            (context, request), IAuthenticator)

    def __call__(self):
        api_method = self.request.form.get('apiMethod')
        if api_method:
            self.request.response.setHeader('Content-type', 'application/json')

        if api_method == 'send_authorization':
            return self.send_authorization()
        if api_method == 'authorize_code':
            return self.authorize_code()
        if api_method == 'login':
            return self.login()
        if api_method == 'set_password':
            return self.set_password()
        if api_method == 'request_country_exception':
            return self.request_country_exception()
        if api_method == 'reset_password':
            return self.reset_password()

        self.request.response.setHeader('X-Theme-Applied', True)
        return self.index()

    @property
    def username(self):
        return self.request.form.get('username')

    def get_country_header(self):
        return (
            self.request.get_header('Cf-Ipcountry') or
            self.request.get_header('Country'))

    def request_country_exception(self):
        # re-check login...
        # verify we get to country block again
        try:
            self.auth.authenticate(
                username=self.username,
                password=self.request.form.get('password'),
                country=self.get_country_header())
            return json.dumps({
                'success': False,
                'message': 'Error authenticating request',
                'messageType': 'error'
            })
        except authentication.AuthenticationCountryBlocked:
            pass

        req_country = self.get_country_header()
        user = api.user.get(username=self.username)
        data = self.auth.issue_country_exception(user, req_country)

        email_subject = "Country block exception request(Site: %s)" % (
            api.portal.get_registry_record('plone.site_title'))

        for admin_user in get_managers():
            if admin_user.getId() == user.getId():
                # it could be an admin, do not allow him to authorize himself
                continue

            admin_email = admin_user.getProperty('email')

            email_data = data.copy()
            email_data.update({
                'admin_name': (admin_user.getProperty('fullname') or
                               admin_user.getUserName()),
                'name': (user.getProperty('fullname') or user.getUserName()),
                'auth_link': '{}/authorize-country-login?code={}&userid={}'.format(
                    self.context.absolute_url(),
                    data['code'],
                    user.getId()
                )
            })
            email_html = '''
<p>
Hello {admin_name},
</p>
<p>
There has been a request to suspend country login restrictions for this site.

<p>
The user requesting this access logged this information:
</p>
<ul>
<li><b>Username</b>: {username}</li>
<li><b>Name</b>: {name}</li>
<li><b>Country</b>: {country}</li>
<li><b>IP</b>: {ip}</li>
<li><b>User agent</b>: {user_agent}</li>
</ul>

<p>The user has 12 hours between after authorization has been giving to login
   with the temporary access</p>

<p>If you'd like to authorize this user, please click this link:
   <a href="{auth_link}">authorize {username}</a>
</p>'''.format(**email_data)
            send_email(admin_email, email_subject, html=email_html)

        return json.dumps({
            'success': True,
            'message': 'Successfully requested country exception.',
            'messageType': 'info'
        })

    def set_password(self):
        # 1. Authenticate user
        authorized, user = self.auth.authenticate(
            username=self.username,
            password=self.request.form.get('existing_password'),
            country=self.get_country_header(),
            login=False)

        if not user or not authorized:
            return json.dumps({
                'success': False,
                'message': 'Existing credentials did not match'
            })

        # 2. Set password
        newpw = self.request.form.get('new_password')

        registration = getToolByName(self.context, 'portal_registration')
        err_str = registration.testPasswordValidity(newpw)

        if err_str is not None:
            return json.dumps({
                'success': False,
                'message': translate(err_str)
            })

        mtool = api.portal.get_tool('portal_membership')
        member = mtool.getMemberById(user.getId())

        self.auth.change_password(member, newpw)

        self.auth.authenticate(
            username=self.username,
            password=newpw,
            country=self.get_country_header(),
            login=True)

        return json.dumps({
            'success': True
        })

    '''
    this function is here since the current
    login form doesn't use PAS for validation.
    '''
    def pwexpiry(self, user):
        pwexpiry_enabled = api.portal.get_registry_record('plone.pwexpiry_enabled', default=False)
        validity_period = api.portal.get_registry_record('plone.pwexpiry_validity_period', default=0)
        if pwexpiry_enabled and validity_period > 0:
            whitelist = api.portal.get_registry_record('plone.pwexpiry_whitelisted_users', default=[])
            whitelisted = whitelist and user.getId() in whitelist
            if not whitelisted:
                password_date = user.getProperty(
                    'password_date',
                    '2000/01/01'
                )
                current_time = DateTime()
                editableUser = api.user.get(username=user.getId())
                if password_date.strftime('%Y/%m/%d') != '2000/01/01':
                    since_last_pw_reset = days_since_event(
                        password_date.asdatetime(),
                        current_time.asdatetime()
                    )

                    '''
                    depending how you intepret the setting, it might make
                    more sense to check if it's <= 0 instead.
                    Leaving as strictly LT for now.
                    '''
                    if validity_period - since_last_pw_reset < 0:
                        # Password has expired
                        editableUser.setMemberProperties({
                            'reset_password_required': True,
                            'reset_password_time': time.time()
                        })
                        return True
                else:
                    editableUser.setMemberProperties({
                        'password_date': current_time
                    })
        return False

    def login(self):
        # double check auth code first
        code = self.request.form.get('code')
        if (self.auth.two_factor_enabled and not
                self.auth.authorize_2factor(self.username, code, 5 * 60)):
            return json.dumps({
                'success': False,
                'message': 'Login failed for username and password.'
            })

        try:
            if hasattr(self.context, 'portal_registry'):
                backend_urls = self.context.portal_registry['plone.backend_url']
                only_allow_login_to_backend_urls = self.context.portal_registry['plone.only_allow_login_to_backend_urls']  # noqa
                portal_url = api.portal.get().absolute_url()
                bad_domain = only_allow_login_to_backend_urls and \
                             len(backend_urls) > 0 and \
                             portal_url.rstrip('/') not in backend_urls and \
                             portal_url.rstrip('/') + '/' not in backend_urls
                if bad_domain:
                    return json.dumps({
                        'success': False,
                        'message': translate(_(
                            u'description_bad_login_domain',
                            default=u'You are attempting to log into this site from the wrong domain; contact your site administrator for assistance.'  # noqa
                        ))
                    })
            authorized, user = self.auth.authenticate(
                username=self.username,
                password=self.request.form.get('password'),
                country=self.get_country_header(),
                login=False)
            if authorized:
                reset_password = user.getProperty(
                    'reset_password_required', False)

                if not self.auth.is_zope_root:
                    reset_password = reset_password | self.pwexpiry(user)

                resp = {
                    'success': True,
                    'resetpassword': reset_password
                }
                if reset_password:
                    resp['message'] = 'Your password has expired.'
                else:
                    authorized, user = self.auth.authenticate(
                        username=self.username,
                        password=self.request.form.get('password'),
                        country=self.get_country_header(),
                        login=True)
                try:
                    with api.env.adopt_user(user=user):
                        resp['authenticator'] = createToken()
                        return json.dumps(resp)
                except Exception:
                    resp['authenticator'] = createToken()
                    return json.dumps(resp)
            else:
                return json.dumps({
                    'success': False,
                    'message': 'Login failed for username and password.'
                })
        except authentication.AuthenticationMaxedLoginAttempts:
            return json.dumps({
                'success': False,
                'message': 'You have reached the have number of login attempts '
                           'for a period.'
            })
        except authentication.AuthenticationUserDisabled:
            return json.dumps({
                'success': False,
                'message': 'User is disabled.'
            })
        except authentication.AuthenticationCountryBlocked:
            return json.dumps({
                'success': True,
                'countryBlocked': True,
                'message': 'User login blocked. The country you are logging '
                           'in from is blocked.'
            })
        except authentication.AuthenticationPasswordResetWindowExpired:
            return json.dumps({
                'success': False,
                'message': 'User login disabled. Password reset request not '
                           'fullfilled in required time period.'
            })

    def authorize_code(self):
        code = self.request.form.get('code')
        if self.authenticator.authorize_2factor(self.username, code):
            return json.dumps({
                'success': True,
                'message': 'Authorization code verified.'
            })
        else:
            return json.dumps({
                'success': False,
                'message': 'Failed to authorize code'
            })

    def send_authorization(self):
        auth_type = self.request.form.get('authType') or 'email'
        if auth_type == 'email':
            self.send_auth_email()
        elif auth_type == 'sms':
            self.send_auth_text()
        return json.dumps({
            'success': True,
            'message': 'Authorization code sent to provided username if '
                       'username exists.'
        })

    def send_auth_email(self):
        email = None
        user = api.user.get(username=self.username)
        if user:
            email = user.getProperty('email')
        if not email:
            return

        code = self.auth.issue_2factor_code(self.username)
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)
        html = """
<p>
    You have requested to authorize access to {title}.
</p>
<p>
    You authorization code is: {code}
</p>""".format(title=site_settings.site_title,
               code=code)

        send_email(
            [email], "Authorization code(%s)" % site_settings.site_title,
            html=html)

    def send_auth_text(self):
        phone = None
        user = api.user.get(username=self.username)
        if user:
            phone = user.getProperty('phone_number')

        if not phone:
            return

        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema,
                                              prefix="plone",
                                              check=False)

        code = self.auth.issue_2factor_code(self.username)

        text_message = '{}: phone verification code: {}'.format(
            site_settings.site_title, code)

        return texting.send(text_message, phone)

    def reset_password(self):
        pw_tool = api.portal.get_tool('portal_password_reset')
        registration = api.portal.get_tool('portal_registration')
        userid = self.request.form.get('userid')
        randomstring = self.request.form.get('code')
        password = self.request.form.get('password')

        err_str = registration.testPasswordValidity(password)
        if err_str is not None:
            return json.dumps({
                'success': False,
                'message': translate(err_str)
            })

        alsoProvides(self.request, IDisableCSRFProtection)
        try:
            pw_tool.resetPassword(userid, randomstring, password)
        except ExpiredRequestError:
            return json.dumps({
                'success': False,
                'message': 'Password reset request has expired'
            })
        except (InvalidRequestError, RuntimeError):
            return json.dumps({
                'success': False,
                'message': 'Password reset request is invalid'
            })

        return json.dumps({
            'success': True,
            'message': 'Password reset successfully'
        })

    def get_registry(self):
        try:
            return getUtility(IRegistry)
        except ComponentLookupError:
            pass

    def get_tool(self, name):
        if self.auth.is_zope_root:
            return getattr(self.context, name, None)
        else:
            return api.portal.get_tool(name)

    def options(self):
        site_url = success_url = self.context.absolute_url()
        if ISiteRoot.providedBy(self.context):
            success_url += '/@@dashboard'
        if 'came_from' in self.request.form:
            came_from = self.request.form['came_from']
            try:
                url_tool = api.portal.get_tool('portal_url')
            except api.exc.CannotGetPortalError:
                url_tool = None
            if (came_from.startswith(site_url) and (
                    not url_tool or url_tool.isURLInPortal(came_from))):
                success_url = came_from

        data = {
            'supportedAuthSchemes': self.authenticator.get_supported_auth_schemes(),
            'twoFactorEnabled': self.authenticator.two_factor_enabled,
            'apiEndpoint': '{}/@@secure-login'.format(site_url),
            'successUrl': success_url
        }
        try:
            data['authenticator'] = createToken()
        except ConnectionStateError:
            # zope root related issue here...
            pass

        username = None
        pwreset = self.request.form.get('pwreset') == 'true'
        if pwreset:
            try:
                user = api.user.get(self.request.form.get('userid'))
                username = user.getUserName()
                data.update({
                    'passwordReset': pwreset,
                    'username': username,
                    'code': self.request.form.get('code'),
                    'userid': self.request.form.get('userid')
                })
            except Exception:
                pwreset = False
        return json.dumps(data)


class LoginExceptionApprovalView(BrowserView):
    implements(ISecureLoginAllowedView)

    message = 'Incorrect code for country exception.'
    success = False

    def __call__(self):
        auth = self.authenticator = getMultiAdapter(
            (self.context, self.request), IAuthenticator)

        userid = self.request.form.get('userid')
        code = self.request.form.get('code')
        if userid and code:
            exc_key = auth.get_country_exception_cache_key(userid)
            try:
                data = cache.get(exc_key)
                if not strings_differ(data['code'], code):
                    timestamp = data.get('timestamp')
                    if timestamp and (time.time() < (timestamp + (12 * 60 * 60))):
                        user = api.user.get(data['userid'])
                        self.message = 'Successfully issued country login exception for {}({}).'.format(  # noqa
                            user.getProperty('fullname') or user.getUserName(),
                            user.getUserName())
                        self.success = True
                        data['granted'] = True
                        data['timestamp'] = time.time()
                        cache.set(exc_key, data, 12 * 60 * 60)
                        self.send_email(data)
            except Exception:
                pass
        return self.index()

    def send_email(self, user, data):
        email_subject = "Country block exception request granted(Site: %s)" % (
            api.portal.get_registry_record('plone.site_title'))

        email = user.getProperty('email')

        email_data = data.copy()
        email_data.update({
            'name': (user.getProperty('fullname') or user.getUserName())
        })
        email_html = '''
<p>
Hello {name},
</p>
<p>
Your request to login from your current location is granted.
<br />
Please login again with the same browser and location you made the request.
If you still experience any problems, please contact your administrator.

<p>
User and location information:
</p>
<ul>
<li><b>Username</b>: {username}</li>
<li><b>Name</b>: {name}</li>
<li><b>Country</b>: {country}</li>
<li><b>IP</b>: {ip}</li>
<li><b>User agent</b>: {user_agent}</li>
</ul>

<p>You have 12 hours to use this granted login exception.</p>
'''.format(**email_data)
        send_email(email, email_subject, html=email_html)



# -*- coding: utf-8 -*-
try:
    from Products.CMFPlone import __version__ as plone_version
except ImportError:
    plone_version = '4'

import logging

from Products.CMFCore.utils import getToolByName
from Products.PlonePAS.Extensions.Install import activatePluginInterfaces
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from pwexpiry_plugin import addPwExpiryPlugin

logger = logging.getLogger('castle.cms.pwexpiry')


def import_various(context):
    """
    Install the PwExpiryPlugin
    """
    portal = context.getSite()

    acl = getToolByName(portal, 'acl_users')
    installed = acl.objectIds()

    if 'pwexpiry' not in installed:
        addPwExpiryPlugin(acl, 'pwexpiry', 'PwExpiry Plugin')
        activatePluginInterfaces(portal, 'pwexpiry')
        for i in range(len(acl.plugins.listPluginIds(IChallengePlugin))):
            acl.plugins.movePluginsUp(IChallengePlugin, ['pwexpiry'])
    else:
        logger.info('pwexpiry already installed')



# -*- coding: utf-8 -*-
from copy import deepcopy
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import json
import unittest
from io import BytesIO

from castle.cms.browser import content
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import (TEST_USER_ID, TEST_USER_NAME, login, logout,
                               setRoles)
from plone.uuid.interfaces import IUUID
from zope.annotation.interfaces import IAnnotations


class TestContent(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_dump_data(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['uid'], IUUID(doc))

    def test_dump_data_with_view(self):
        folder = api.content.create(
            type='Folder',
            id='file-repository',
            container=self.portal)
        doc = api.content.create(
            type='File',
            id='fi',
            container=folder)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['url'], doc.absolute_url() + '/view')

    def test_workflow_transition(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        self.request.form.update({
            'transition_id': 'publish',
            'action': 'transition'
        })
        wf = content.Workflow(doc, self.request)
        wf()
        self.assertEqual(api.content.get_state(doc), 'published')

    def test_publish_content_twice(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        pc = content.PublishContent(doc, self.request)
        pc()
        # second once used to cause InvalidParameterError
        pc()
        self.assertEqual(api.content.get_state(doc), 'published')

    def test_check(self):
        self.request.form.update({
            'action': 'check',
            'selectedType[typeId]': 'Folder',
            'basePath': '/',
            'id': 'foobar'
        })
        cc = content.Creator(self.portal, self.request)
        data = cc()
        self.assertEqual(json.loads(data)['valid'], True)

    def test_upload(self):
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024 * 5,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue('id' in data)
        self.assertTrue(data['success'])
        self.request.form.update({
            'id': data['id']
        })

        for idx in range(4):
            self.request.form.update({
                'action': 'chunk-upload',
                'chunk': str(idx + 2),
                'file': BytesIO('X' * 1024)
            })
            cc = content.Creator(self.portal, self.request)
            data = json.loads(cc())
            self.assertTrue(data['success'])

        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)

        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024 * 5)
        return fileOb

    def test_update_upload(self):
        fileOb = self.test_upload()
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('U' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'file'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)
        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'U' * 1024)

    def test_tmp_upload(self):
        fileOb = self.test_upload()
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('T' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'tmp_blarg'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertTrue(data['valid'])
        self.assertTrue('url' in data)
        fileOb = api.content.get(path='/file-repository/foobar.bin')

        # should not change!
        self.assertEquals(fileOb.file.data, 'X' * 1024 * 5)

        # tmp file should be uploaded
        annotations = IAnnotations(fileOb)
        tmp_files = annotations['_tmp_files']
        self.assertTrue('tmp_blarg' in tmp_files)

    def test_create_folders(self):
        self.request.form.update({
            'action': 'create',
            'basePath': 'foo/bar',
            'id': 'foobar',
            'title': 'Foobar',
            'selectedType[id]': 'Document',
            'transitionTo': 'publish'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        ob = api.content.get(path='/foo/bar/foobar')
        self.assertTrue(ob is not None)
        self.assertEquals(
            data['base_url'], 'http://nohost/plone/foo/bar/foobar')
        self.assertEquals(
            api.content.get_state(ob), 'published')

    def test_save_file_saves_custom_field_values(self):

        # add new field
        registry = getUtility(IRegistry)
        fields = deepcopy(registry['castle.file_upload_fields'])
        for f in fields:
            f['required'] = unicode(f['required']).lower()
        fields.append({
            u'name': u'foobar',
            u'label': u'Foobar',
            u'widget': u'text',
            u'required': u'false',
            u'for-file-types': u'*'
        })
        registry['castle.file_upload_fields'] = fields

        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin',
            'title': 'Foobar',
            'foobar': 'Some value here'
        })
        cc = content.Creator(self.portal, self.request)
        cc()

        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024)
        self.assertEquals(fileOb.foobar, 'Some value here')
        return fileOb


class TestContentAccess(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        logout()

    def test_accees_denied(self):
        content.Creator(self.portal, self.request)()
        self.assertEquals(self.request.response.getStatus(), 403)

    def test_modify_content_upload_permission(self):
        api.user.create(
            email='foo@bar.com', username='foobar',
            password='foobar', roles=('Member',))
        setRoles(self.portal, 'foobar', (
            'Member', 'Contributor', 'Reviewer'))
        api.user.create(
            email='foo2@bar.com', username='foobar2',
            password='foobar2', roles=('Member',))
        setRoles(self.portal, 'foobar2', (
            'Member', 'Reviewer'))

        login(self.portal, 'foobar')
        self.request.form.update({
            'action': 'chunk-upload',
            'chunk': '1',
            'chunkSize': 1024,
            'totalSize': 1024,
            'file': BytesIO('X' * 1024),
            'name': 'foobar.bin'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        fileOb = api.content.get(path='/file-repository/foobar.bin')
        self.assertEquals(fileOb.file.data, 'X' * 1024)

        # and publish the sucker
        api.content.transition(fileOb, 'publish')

        logout()
        login(self.portal, 'foobar2')

        self.request.form.update({
            'action': 'chunk-upload',
            'file': BytesIO('U' * 1024),
            'name': 'foobar2.bin',
            'content': IUUID(fileOb),
            'field': 'file'
        })
        cc = content.Creator(self.portal, self.request)
        data = json.loads(cc())
        self.assertFalse(data['success'])


class TestPageLayoutSelector(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_get_front_page(self):
        view = content.PageLayoutSelector(
            self.portal['front-page'], self.request)
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'frontpage.html')
        self.assertEqual(result['section_layout'], None)

    def test_set_front_page(self):
        view = content.PageLayoutSelector(
            self.portal['front-page'], self.request)
        self.request.form.update({
            'data': json.dumps({
                'page_layout': 'foobar.html',
                'section_layout': 'foobar.html'
            }),
            'action': 'save'
        })
        view()

        # check results...
        del self.request.form['action']
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'foobar.html')
        self.assertEqual(result['section_layout'], 'foobar.html')

        # also check that the registry entry value set since
        # front page is default page for site
        self.assertEqual(
            self.portal.portal_registry['castle.cms.default_layout'],
            'foobar.html')



# -*- coding: utf-8 -*-
import os.path

from castle.cms.fragments import FragmentsDirectory
from castle.cms.fragments.interfaces import IFragmentsDirectory

from zope.interface import Interface
from zope.component import getGlobalSiteManager
from zope.configuration.exceptions import ConfigurationError
from zope.configuration.fields import PythonIdentifier
from zope.schema import TextLine


class IFragmentDirectoryDirective(Interface):

    directory = TextLine(
        title=u'Directory path',
        description=u'Path relative to the package.',
        required=True
        )

    name = PythonIdentifier(
        title=u'Name',
        description=u'Name of the directory. If not specified, the name of '
                    u'the current package is used.',
        required=True,
        )


def registerFragmentDirectory(_context, directory, name=None, type=None):

    """

    Register a zcml fragment folder declaration

    This is shamelessly adapted from the zope plone folder declaration
    at https://github.com/plone/plone.resource/blob/master/plone/resource/zcml.py

    """  # noqa
    if _context.package and os.path.isabs(directory):
        raise ConfigurationError('Resource directories in distributions must '
                                 'be specified as relative paths.')
    elif _context.package:
        directory = _context.path(directory)
    elif not _context.package and not os.path.isabs(directory):
        raise ConfigurationError('Global resource directories must be '
                                 'specified as absolute paths.')

    if '..' in directory.split('/'):
        raise ConfigurationError('Traversing to parent directories '
                                 'via .. is not allowed.')
    if not os.path.exists(directory):
        raise IOError('Directory not found: %s' % directory)

    if name is None and _context.package:
        name = _context.package.__name__

    fd = FragmentsDirectory()
    fd.directory_path = directory

    gsm = getGlobalSiteManager()
    gsm.registerUtility(fd, IFragmentsDirectory, name)



from castle.cms.interfaces import ICastleApplication
from plone import api
from Products.Five import BrowserView


class ErrorView(BrowserView):

    @property
    def is_zope_root(self):
        return ICastleApplication.providedBy(self.context)

    @property
    def logged_in(self):
        try:
            return not api.user.is_anonymous()
        except Exception:
            return False

    def format_error_message(self):
        return repr(self.context)

    def __call__(self):
        return self.index()



from castle.cms.cron.utils import login_as_admin
from castle.cms.cron.utils import setup_site
from plone.app.blocks import tiles
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.app.blocks.utils import getLayout
from plone.app.uuid.utils import uuidToCatalogBrain
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from repoze.xmliter.utils import getHTMLSerializer
from unidecode import unidecode
from zope.globalrequest import getRequest

import logging
import requests
import argparse
import datetime


logger = logging.getLogger(__name__)


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--site-id', dest='site_id', default='Castle')
args, _ = parser.parse_known_args()


class BadResponse(object):
    status_code = 500


_safe = [
    'https://twitter.com/intent/tweet',
    'https://www.facebook.com/sharer/sharer.php',
    'http://www.addthis.com',
    'javascript:'
]

_known_bad = [
    'http://www.captcha.net',
    'https://elements.psu.edu',
    'https://supervisord.org',
]


def find_url(ob, url):
    if 'tmbc.me' in url:
        return False

    for bad in _known_bad:
        if url.startswith(bad):
            return False

    for safe in _safe:
        if safe in url:
            return True

    if 'resolveuid/' in url:
        # check if object...
        uid = url.split('resolveuid/')[-1]
        uid = uid.split('/')[0]
        return uuidToCatalogBrain(uid) is not None
    elif (('https://' in url or 'http://' in url) and
            'http://nohost' not in url):
        try:
            print('checking ' + url)
            resp = requests.get(url)
        except Exception:
            resp = BadResponse()
        return resp.status_code == 200
    else:
        if 'http://nohost' in url:
            url = url.replace('http://nohost', '')
        url = url.replace('%20', ' ').split(
            '?')[0].split('#')[0].split('/@@images')[0]
        try:
            return ob.restrictedTraverse(str(url), None) is not None
        except Exception:
            return False


def find_broken(site):
    setup_site(site)
    catalog = site.portal_catalog

    broken = []
    good_urls = []
    checked_urls = []

    req = getRequest()
    for brain in catalog(object_provides=ILayoutAware.__identifier__):
        ob = brain.getObject()
        layout = getLayout(ob)
        dom = getHTMLSerializer(layout)
        tiles.renderTiles(req, dom.tree, ob.absolute_url() + '/layout_view')
        root = dom.tree.getroot()
        for anchor in root.cssselect('a'):
            if not anchor.attrib.get('href'):
                continue
            url = anchor.attrib['href']
            if (url[0] == '#' or url.startswith('data:') or
                    url.startswith('mailto:')):
                continue
            if url in good_urls:
                continue
            if url in checked_urls:
                print('skipping already checked {}'.format(url))
                continue
            checked_urls.append(url)
            if find_url(ob, url):
                good_urls.append(url)
            else:
                try:
                    text = unidecode(anchor.text_content())
                except Exception:
                    text = ''
                result = '{} linking to broken -> {}({})'.format(
                    brain.getPath(),
                    url,
                    text
                )
                broken.append(result)
                print(result)

        for img in root.cssselect('img'):
            if not img.attrib.get('src'):
                continue
            url = img.attrib['src']
            if url[0] == '#' or url.startswith('data:'):
                continue
            if url in checked_urls:
                print('skipping already checked {}'.format(url))
                continue
            checked_urls.append(url)
            if find_url(ob, url):
                good_urls.append(url)
            else:
                result = '{} linking to broken image -> {}'.format(
                    brain.getPath(),
                    url
                )
                broken.append(result)
                print(result)

    now = datetime.datetime.now()
    filename = 'broken-links-{}.txt'.format(now.isoformat())
    fi = open(filename, 'w')
    fi.write('\n'.join(broken))
    fi.close()


if __name__ == '__main__':
    login_as_admin(app)  # noqa
    site = app[args.site_id]  # noqa
    if IPloneSiteRoot.providedBy(site):
        try:
            find_broken(site)
        except Exception:
            logger.error('Encountered error %s' % site, exc_info=True)
    else:
            logger.error('%s is not a site' % site)



from castle.cms.browser.survey import ICastleSurvey
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class SurveyControlPanelForm(RegistryEditForm):
    schema = ICastleSurvey
    id = "SurveyControlPanel"
    label = u'GovSurvey.us Settings'
    description = "Engage your website vistors and request feedback and improvements or just ask some general questions! Signup with https://GovSurvey.us"  # noqa

    control_panel_view = '@@survey-controlpanel'


class SurveyControlPanel(ControlPanelFormWrapper):
    form = SurveyControlPanelForm
    index = ViewPageTemplateFile('templates/survey.pt')



from AccessControl import AuthEncoding
from castle.cms.interfaces.passwordvalidation import ICustomPasswordValidator
from plone import api
from zope.interface import implementer

from .config import _


@implementer(ICustomPasswordValidator)
class PasswordHistoryValidator(object):
    """ Checks if the password has already been used
    by iterating over the memberdata property password_history_size.
    """

    def __init__(self, context):
        self.context = context

    def validate(self, password, data):
        """
        Password validation method
        """

        # Permit empty passwords here to allow registration links.
        # Plone will force the user to set one.
        if password is None:
            return None

        if api.user.is_anonymous():
            # No check for registrations.
            return None

        pwexpiry_enabled = api.portal.get_registry_record(
            'plone.pwexpiry_enabled'
        )
        max_history_pws = api.portal.get_registry_record(
            'plone.pwexpiry_password_history_size'
        )
        if max_history_pws == 0 or not pwexpiry_enabled:
            # max_history_pws has been disabled.
            return None

        user = api.user.get_current()

        # Ignore whitelisted
        whitelisted = api.portal.get_registry_record(
            'plone.pwexpiry_whitelisted_users'
        )
        if whitelisted and user.getId() in whitelisted:
            return None

        pw_history = user.getProperty('password_history', [])

        for old_pw in pw_history[-max_history_pws:]:
            if AuthEncoding.pw_validate(old_pw, str(password)):
                return _(
                    u'info_reused_pw',
                    default=u'This password has been used already.'
                )

        # All fine
        return None



from Products.Five.browser import BrowserView
from elasticsearch import Elasticsearch
import elasticsearch
import subprocess
import redis
import celery
import celery.bin.base
import celery.bin.celery
import celery.platforms


class StatusView(BrowserView):

    def docsplit(self):
        try:
            subprocess.check_call('docsplit')
            x = True, 'ok'
        except OSError as e:
            x = False, str(e)
        return x

    def redis(self):
        r = redis.Redis("localhost")
        try:
            r.client_list()
            x = True, 'ok'
        except redis.ConnectionError as e:
            x = False, str(e)
        return x

    def celery(self):
        celery.Celery('tasks', broker='redis://')
        status = celery.bin.celery.CeleryCommand.commands['status']()
        status.app = status.get_app()
        try:
            status.run()
            x = True, 'ok'
        except celery.bin.base.Error as e:
            x = False, str(e)
        except redis.ConnectionError:
            x = False, 'Cannot connect to Redis'
        return x

    def elasticsearch(self):
        es = Elasticsearch()
        try:
            (es.cluster.health())
            x = True, 'ok'
        except elasticsearch.ConnectionError as e:
            x = False, str(e)
        return x



from zope.interface import Interface
from zope import schema
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import form


class ICastleSurvey(Interface):
    survey_api_url = schema.TextLine(
        title=u'GovSurvey API URL',
        description=u'GovSurvey.us API URL; https://govsurvey.us for more information.',
        required=False
    )

    survey_account_id = schema.TextLine(
        title=u'GovSurvey API Account ID',
        description=u'Account ID from your account on the GovSurvey system API',
        required=False
    )


class CastleSurvey(form.Form):
    label = u"Survey"
    description = u"A GovSurvey survey"
    formErrorsMessage = 'There were errors in a Castle Survey'
    ignoreContext = True
    schema = ICastleSurvey
    template = ViewPageTemplateFile("templates/survey.pt")



from AccessControl.Permissions import add_user_folders
from Products.PluggableAuthService.PluggableAuthService import \
    registerMultiPlugin
import pwexpiry_plugin


def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    registerMultiPlugin(pwexpiry_plugin.PwExpiryPlugin.meta_type)
    context.registerClass(
        pwexpiry_plugin.PwExpiryPlugin,
        permission=add_user_folders,
        constructors=(
            pwexpiry_plugin.manage_addPwExpiryPluginForm,
            pwexpiry_plugin.addPwExpiryPlugin
        ),
        visibility=None
    )



import logging

from AccessControl import AuthEncoding
from castle.cms.pwexpiry.events import (InvalidPasswordEntered,
                                        ValidPasswordEntered)
from castle.cms.interfaces.passwordvalidation import ICustomPasswordValidator
from plone import api
from Products.CMFPlone.RegistrationTool import RegistrationTool
from Products.PluggableAuthService.plugins.ZODBUserManager import \
    ZODBUserManager
from zope.component import getAdapters
from zope.event import notify

from hashlib import sha1 as sha


logger = logging.getLogger(__file__)


original_testPasswordValidity = RegistrationTool.testPasswordValidity


def extended_testPasswordValidity(self, password, confirm=None, data=None):
    """
    Patching the standard Plone's testPasswordValidity method to
    enable registering a custom password validator.
    """
    validators = getAdapters((self,), ICustomPasswordValidator)
    for name, validator in validators:
        result = validator.validate(password, data)
        if result:
            return result

    original = original_testPasswordValidity(self, password, confirm)
    if original:
        return original

    return None


RegistrationTool.testPasswordValidity = extended_testPasswordValidity


logger.info(
    "Patching Products.CMFDefault.RegistrationTool.testPasswordValidity"
)

ZODBUserManager.original_authenticateCredentials = ZODBUserManager.authenticateCredentials


def authenticateCredentials(self, credentials):
    """ See IAuthenticationPlugin.

    o We expect the credentials to be those returned by
      ILoginPasswordExtractionPlugin.
    """
    login = credentials.get('login')
    password = credentials.get('password')

    if login is None or password is None:
        return None

    # Do we have a link between login and userid?  Do NOT fall
    # back to using the login as userid when there is no match, as
    # that gives a high chance of seeming to log in successfully,
    # but in reality failing.
    userid = self._login_to_userid.get(login)
    if userid is None:
        # Someone may be logging in with a userid instead of a
        # login name and the two are not the same.  We could try
        # turning those around, but really we should just fail.
        #
        # userid = login
        # login = self._userid_to_login.get(userid)
        # if login is None:
        #     return None
        return None

    reference = self._user_passwords.get(userid)

    if reference is None:
        return None

    is_authenticated = False
    if AuthEncoding.is_encrypted(reference):
        if AuthEncoding.pw_validate(reference, password):
            is_authenticated = True

    if not is_authenticated:
        # Support previous naive behavior
        digested = sha(password).hexdigest()

        if reference == digested:
            is_authenticated = True

    if is_authenticated:
        try:
            user = api.user.get(username=login)
        except Exception:
            return userid, login

        event = ValidPasswordEntered(user)
        notify(event)
        return userid, login
    else:
        try:
            user = api.user.get(username=login)
        except Exception:
            return None

        event = InvalidPasswordEntered(user)
        notify(event)
        return None


ZODBUserManager.authenticateCredentials = authenticateCredentials
logger.info("Patching Products.PluggableAuthService.plugins.ZODBUserManager."
            "ZODBUserManager.authenticateCredentials")



# -*- coding: utf-8 -*-

import time

from AccessControl import AuthEncoding, ClassSecurityInfo, Unauthorized
from castle.cms.pwexpiry.config import _
from castle.cms.pwexpiry.utils import days_since_event
from DateTime import DateTime
from Globals import InitializeClass
from plone import api
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PlonePAS.interfaces.plugins import IUserManagement
from Products.PluggableAuthService.interfaces.plugins import (IAuthenticationPlugin,
                                                              IChallengePlugin)
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import implements

manage_addPwExpiryPluginForm = PageTemplateFile(
    'www/addPwExpiryPlugin',
    globals(), __name__='manage_addPwExpiryPlugin'
)


def addPwExpiryPlugin(self, id, title='', REQUEST=None):
    """
    Add PwExpiry plugin
    """
    o = PwExpiryPlugin(id, title)
    self._setObject(o.getId(), o)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
            '%s/manage_main'
            '?manage_tabs_message=PwExpiry+Plugin+added.' %
            self.absolute_url()
        )


class PwExpiryPlugin(BasePlugin):
    """
    Password expiry plugin
    """
    meta_type = 'Password Expiry Plugin'
    security = ClassSecurityInfo()
    implements(IAuthenticationPlugin, IChallengePlugin, IUserManagement)

    def __init__(self, id, title=None):
        self._setId(id)
        self.title = title

    # IAuthenticationPlugin implementation
    security.declarePrivate('authenticateCredentials')

    def authenticateCredentials(self, credentials):
        """
        Check if the user.password_date is older than validity_period.
        If validity_period is 0, skip the check
        """
        login = credentials.get('login')
        if not login:
            return None

        self._invalidatePrincipalCache(login)
        user = api.user.get(username=login)
        if not user:
            return None

        pwexpiry_enabled = api.portal.get_registry_record('plone.pwexpiry_enabled', default=False)
        validity_period = api.portal.get_registry_record('plone.pwexpiry_validity_period', default=0)
        if validity_period == 0 or not pwexpiry_enabled:
            return None

        # Ignore whitelisted
        whitelisted = api.portal.get_registry_record(
            'plone.pwexpiry_whitelisted_users',
            default=[]
        )
        if whitelisted and user.getId() in whitelisted:
            return None

        password_date = user.getProperty('password_date', '2000/01/01')
        if str(password_date) != '2000/01/01':
            current_time = DateTime()
            since_last_pw_reset = days_since_event(password_date.asdatetime(),
                                                   current_time.asdatetime())
            # Password has expired
            if validity_period - since_last_pw_reset < 0:
                user.setMemberProperties({
                    'reset_password_required': True,
                    'reset_password_time': time.time()
                })
                self.REQUEST.RESPONSE.setHeader('user_expired', user.getId())
                raise Unauthorized
        else:
            user.setMemberProperties({
                'password_date': current_time
            })
        return None

    # IChallengePlugin implementation
    security.declarePrivate('challenge')
    def challenge(self, request, response, **kw):  # noqa
        """
        Challenge the user for credentials
        """
        user_expired = response.getHeader('user_expired')
        if user_expired:
            portal_url = api.portal.get_tool(name='portal_url')()
            IStatusMessage(request).add(
                _(u'Your password has expired.'), type='error'
            )
            response.redirect(
                '%s/mail_password_form?userid=%s' % (portal_url, user_expired),
                lock=1
            )
            return 1
        return 0

    # IUserManagement implementation
    security.declarePrivate('doChangeUser')
    def doChangeUser(self, principal_id, password):  # noqa
        """
        Update user's password date and store passwords history.
        """
        user = api.user.get(username=principal_id)
        portal = api.portal.get()
        current_time = portal.ZopeTime()
        user.setMemberProperties({'password_date': current_time})
        self._invalidatePrincipalCache(principal_id)

        # Remember passwords here
        max_history_pws = api.portal.get_registry_record(
            'plone.pwexpiry_password_history_size',
            default=0
        )

        if max_history_pws == 0:
            # disabled, return here.
            return

        enc_pw = password
        if not AuthEncoding.is_encrypted(enc_pw):
            enc_pw = AuthEncoding.pw_encrypt(enc_pw)

        pw_history = list(user.getProperty('password_history', []))
        pw_history.append(enc_pw)
        if len(pw_history) > max_history_pws:
            # Truncate the history
            pw_history = pw_history[-max_history_pws:]

        user.setMemberProperties({'password_history': pw_history})


InitializeClass(PwExpiryPlugin)



import logging
from time import time

from AccessControl import getSecurityManager
from Acquisition import aq_parent
from castle.cms import authentication, cache
from castle.cms.interfaces import ICastleApplication
from collective.elasticsearch.es import ElasticSearchCatalog  # noqa
from OFS.CopySupport import (CopyError, _cb_decode, _cb_encode, eInvalid,
                             eNoData)
from plone import api
from plone.keyring.interfaces import IKeyManager
from plone.registry.interfaces import IRegistry
from plone.session import tktauth
from plone.transformchain.interfaces import ITransform
from Products.CMFPlone.interfaces import ITinyMCESchema
from ZODB.POSException import ConnectionStateError
from zope.component import getGlobalSiteManager, getUtility, queryUtility
from zope.interface import implementer

logger = logging.getLogger('castle.cms')


def HideSiteLayoutFields_update(self):
    """
    we don't want to hide these fields
    """
    return


_rich_text_widget_types = (
    'plone_app_z3cform_wysiwyg_widget_WysiwygWidget',
    'plone_app_z3cform_wysiwyg_widget_WysiwygFieldWidget',
    'plone_app_widgets_dx_RichTextWidget',
    'plone_app_z3cform_widget_RichTextFieldWidget',

)


def MosaicRegistry_parseRegistry(self):
    cache_key = '%s-mosaic-registry' % '/'.join(
        api.portal.get().getPhysicalPath()[1:])
    try:
        result = cache.get(cache_key)
    except KeyError:
        result = self._old_parseRegistry()
        cache.set(cache_key, result, 60 * 2)  # cache for 2 minutes

    registry = getUtility(IRegistry)
    settings = registry.forInterface(
        ITinyMCESchema, prefix="plone", check=False)
    if settings.libraries_spellchecker_choice != 'AtD':
        return result

    # add atd config to toolbar dynamically
    mos_settings = result['plone']['app']['mosaic']
    mos_settings['richtext_toolbar']['AtD'] = {
        'category': u'actions',
        'name': u'toolbar-AtD',
        'weight': 0,
        'favorite': False,
        'label': u'After the deadline',
        'action': u'AtD',
        'icon': False
    }
    for widget_type in _rich_text_widget_types:
        mos_settings['widget_actions'][widget_type]['actions'].append('toolbar-AtD')  # noqa
    mos_settings['structure_tiles']['text']['available_actions'].append('toolbar-AtD')  # noqa
    mos_settings['app_tiles']['plone_app_standardtiles_rawhtml']['available_actions'].append('toolbar-AtD')  # noqa
    return result


if not hasattr(ElasticSearchCatalog, 'original_searchResults'):
    ElasticSearchCatalog.original_searchResults = ElasticSearchCatalog.searchResults  # noqa

    def searchResultsTrashed(self, REQUEST=None, check_perms=False, **kw):
        if 'trashed' not in kw:
            kw['trashed'] = False
        return self.original_searchResults(REQUEST, check_perms, **kw)
    ElasticSearchCatalog.searchResults = searchResultsTrashed


def Content_addCreator(self, creator=None):
    """ Add creator to Dublin Core creators.
    """
    if len(self.creators) > 0:
        # do not add creator if one is already set
        return

    if creator is None:
        user = getSecurityManager().getUser()
        creator = user and user.getId()

    # call self.listCreators() to make sure self.creators exists
    if creator and creator not in self.listCreators():
        self.creators = self.creators + (creator, )


def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
    """
    override normal paste action to see if we're looking at the paste
    data being stored in cache
    """
    if cb_copy_data is not None:
        cp = cb_copy_data
    elif REQUEST is not None and '__cp' in REQUEST:
        cp = REQUEST['__cp']
    else:
        cp = None
    if cp is None:
        raise CopyError(eNoData)

    try:
        op, mdatas = _cb_decode(cp)
    except Exception:
        raise CopyError(eInvalid)

    try:
        if mdatas[0][0].startswith('cache:'):
            cache_key = mdatas[0][0].replace('cache:', '')
            new_mdatas = cache.get(cache_key)
            cdata = _cb_encode((op, new_mdatas))
            return self._old_manage_pasteObjects(cdata)
    except IndexError:
        pass
    return self._old_manage_pasteObjects(cb_copy_data, REQUEST)


@implementer(ITransform)
class NoopTransform(object):

    order = 8800

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def transformString(self, result, encoding):
        return None
    transformUnicode = transformIterable = transformString


def AppInitializer_initialize(self):
    self._old_initialize()
    app = self.app[0]
    authentication.install_acl_users(app, self.commit)


def SessionPlugin_validateTicket(self, ticket, now=None):
    if now is None:
        now = time()
    if self._shared_secret is not None:
        ticket_data = tktauth.validateTicket(
            self._shared_secret,
            ticket,
            timeout=self.timeout,
            now=now,
            mod_auth_tkt=self.mod_auth_tkt
        )
    else:
        ticket_data = None
        parent = aq_parent(aq_parent(self))

        is_root = ICastleApplication.providedBy(parent)
        if is_root:
            manager = getGlobalSiteManager().queryUtility(IKeyManager)
        else:
            manager = queryUtility(IKeyManager)

        if manager is None:
            return None

        try:
            for secret in manager[u"_system"]:
                if secret is None:
                    continue
                ticket_data = tktauth.validateTicket(
                    secret,
                    ticket,
                    timeout=self.timeout,
                    now=now,
                    mod_auth_tkt=self.mod_auth_tkt
                )
                if ticket_data is not None:
                    break
        except ConnectionStateError:
            logger.warning(
                'Connection state error, swallowing', exc_info=True)
    return ticket_data



from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.resources.browser.cook import cookWhenChangingSettings
from zope.component import getUtility

PROFILE_ID = 'profile-castle.cms.upgrades:2_2_0'


def upgrade(context, logger=None):
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile('profile-Products.PloneKeywordManager:default')
    setup.runAllImportStepsFromProfile(PROFILE_ID)
    cookWhenChangingSettings(api.portal.get())

    # upgrade upload fields
    registry = getUtility(IRegistry)

    if not registry.get('castle.file_upload_fields', None):
        # can be None or [] so we don't do an `is None` check here
        required_upload_fields = registry.get(
            'castle.required_file_upload_fields', []) or []
        registry['castle.file_upload_fields'] = [{
            u'name': u'title',
            u'label': u'Title',
            u'widget': u'text',
            u'required': unicode('title' in required_upload_fields).lower(),
            u'for-file-types': u'*'
        }, {
            u'name': u'description',
            u'label': u'Summary',
            u'widget': u'textarea',
            u'required': unicode('description' in required_upload_fields).lower(),
            u'for-file-types': u'*'
        }, {
            u'name': u'tags',
            u'label': u'Tags',
            u'widget': u'tags',
            u'required': unicode('tags' in required_upload_fields).lower(),
            u'for-file-types': u'*'
        }, {
            u'name': u'youtube_url',
            u'label': u'Youtube URL',
            u'widget': u'text',
            u'required': unicode('youtube_url' in required_upload_fields).lower(),
            u'for-file-types': u'video'
        }]
        if 'castle.required_file_upload_fields' in registry.records._fields:
            del registry.records['castle.required_file_upload_fields']



from Acquisition import aq_inner
from castle.cms import texting
from castle.cms import subscribe
from castle.cms.browser.utils import Utils
from castle.cms.constants import ALL_SUBSCRIBERS
from castle.cms.interfaces import IAnnouncementData
from castle.cms.tasks import send_email
from castle.cms.tasks import send_email_to_subscribers
from castle.cms.widgets import AjaxSelectFieldWidget
from castle.cms.widgets import SelectFieldWidget
from castle.cms.widgets import TinyMCETextFieldWidget
from plone import api
from plone.app.registry.browser import controlpanel
from plone.app.textfield import RichText
from plone.app.textfield.value import RichTextValue
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from plone.outputfilters import apply_filters
from plone.outputfilters.interfaces import IFilter
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import form
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.component import getAdapters
from zope.component import getUtility
from zope.interface import Invalid
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

reg_key = 'castle.subscriber_categories'


class AnnouncementsControlPanelForm(controlpanel.RegistryEditForm):
    schema = IAnnouncementData
    schema_prefix = 'castle'
    id = "AnnouncementsControlPanel"
    label = u"Announcements"
    description = ""

    def updateFields(self):
        super(AnnouncementsControlPanelForm, self).updateFields()
        self.fields['site_announcement'].widgetFactory = TinyMCETextFieldWidget


class ISendEmailUsersForm(model.Schema):
    subject = schema.ASCIILine(
        title=u'Subject')

    send_from = schema.TextLine(
        title=u'Custom FROM address',
        required=False,
    )

    directives.widget(
        'send_to_groups',
        AjaxSelectFieldWidget,
        vocabulary='plone.app.vocabularies.Groups'
    )

    send_to_groups = schema.List(
        title=u'Send to groups',
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Groups'
        ),
        required=False
    )

    directives.widget(
        'send_to_users',
        AjaxSelectFieldWidget,
        vocabulary='plone.app.vocabularies.Users'
    )

    send_to_users = schema.List(
        title=u'Send to users',
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Users'
        ),
        required=False
    )

    send_to_custom = schema.List(
        title=u'To(additional)',
        description=u'Additional email addresses, one per line, to '
                    u'send emails to.',
        value_type=schema.TextLine(),
        required=False
    )

    body = RichText(
        title=u'Body',
        description=u'Message body',
        default_mime_type='text/html',
        output_mime_type='text/html',
        allowed_mime_types=('text/html',),
        required=True)


class SendEmailUsersForm(AutoExtensibleForm, form.Form):
    schema = ISendEmailUsersForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='send')
    def handle_send(self, action):
        data, errors = self.extractData()
        if not errors:
            addresses = data['send_to_custom'] or []
            for group in data['send_to_groups'] or []:
                for user in api.user.get_users(groupname=group):
                    email = user.getProperty('email')
                    if email:
                        addresses.append(email)
            for username in data['send_to_users'] or []:
                user = api.user.get(username=username)
                if user:
                    email = user.getProperty('email')
                    if email:
                        addresses.append(email)

            try:
                sender = data['send_from']
            except Exception:
                sender = None

            utils = Utils(self.context, self.request)
            public_url = utils.get_public_url()
            html = data['body'].output

            filters = [f for _, f in getAdapters((self.context, self.request), IFilter)]
            html = apply_filters(filters, html)
            html = html.replace(self.context.absolute_url(), public_url.encode('utf8'))

            send_email.delay(list(set(addresses)), data['subject'], html=html, sender=sender)
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class ISendEmailSubscribersForm(model.Schema):
    subject = schema.ASCIILine(
        title=u'Subject')

    directives.widget('send_to_categories', SelectFieldWidget)
    send_to_categories = schema.List(
        title=u'Send to categories',
        required=False,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )

    send_from = schema.TextLine(
        title=u'Custom FROM address',
        required=False,
    )

    body = RichText(
        title=u'Body',
        description=u'Message body',
        default_mime_type='text/html',
        output_mime_type='text/html',
        allowed_mime_types=('text/html',),
        required=True,
        default=RichTextValue(
            u'<p></p><p></p>'
            u'<p><a href="{{change_url}}">Change your subscription settings</a></p>'
            u'<p><a href="{{unsubscribe_url}}">Unsubscribe from these messages</a></p>',
            'text/html', 'text/html'))


class SendEmailSubscribersForm(AutoExtensibleForm, form.Form):
    schema = ISendEmailSubscribersForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='send2')
    def handle_send2(self, action):
        data, errors = self.extractData()
        if not errors:
            utils = Utils(self.context, self.request)
            public_url = utils.get_public_url()
            html = data['body'].output

            filters = [f for _, f in getAdapters((self.context, self.request), IFilter)]
            html = apply_filters(filters, html)
            html = html.replace(self.context.absolute_url(), public_url.encode('utf8'))

            categories = set()
            if 'form.widgets.send_to_categories' in self.request.form:
                categories = set(data['send_to_categories'])

            sender = None
            if 'form.widgets.send_from' in self.request.form:
                sender = data['send_from']

            send_email_to_subscribers.delay(data['subject'], html=html, categories=categories, sender=sender)

            api.portal.show_message(
                'Sending emails', request=self.request, type='info')
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class ISendTextForm(model.Schema):

    text = schema.Text(
        title=u'Text message')


class SendTextForm(AutoExtensibleForm, form.Form):

    schema = ISendTextForm

    ignoreContext = True

    @button.buttonAndHandler(u'Send', name='text')
    def handle_text(self, action):
        data, errors = self.extractData()
        if not errors:
            texting.send(data['text'], ALL_SUBSCRIBERS)
            api.portal.show_message(
                'Text message sent', request=self.request, type='info')
            self.request.response.redirect('%s/@@announcements-controlpanel' % (
                self.context.absolute_url()))


class IExportSubscribersForm(model.Schema):
    directives.widget('export_categories', SelectFieldWidget)
    export_categories = schema.List(
        title=u'Subscribers to export',
        description=u'Leave Empty For All Subscribers',
        required=False,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )


class ExportSubscribersForm(AutoExtensibleForm, form.Form):
    schema = IExportSubscribersForm

    ignoreContext = True

    @button.buttonAndHandler(u'Export', name='export')
    def handle_export(self, action):
        data, errors = self.extractData()
        if not errors:
            response = self.request.response
            cd = 'attachment; filename=subscribers.csv'
            response.setHeader('Content-Disposition', cd)
            fields = ['name', 'email', 'phone_number', 'phone_number_confirmed',
                      'confirmed', 'code', 'created', 'captcha', 'categories']
            responsebody = ','.join(fields)
            categories = set()
            if 'form.widgets.export_categories' in self.request.form:
                if data['export_categories'] != u'':
                    categories = set(data['export_categories'])
            check_categories = (categories is not None and len(categories) != 0)
            for subscriber in subscribe.all():
                if check_categories:
                    if ('categories' in subscriber and len(subscriber['categories']) > 0):
                        if len(categories.intersection(subscriber['categories'])) == 0:
                            continue
                row = []
                for key in fields:
                    if subscriber.get(key) is None:
                        row.append('')
                    elif isinstance(subscriber.get(key), list):
                        row.append('"' + ';'.join(subscriber.get(key)) + '"')
                    else:
                        row.append(str(subscriber.get(key)))
                responsebody += '\n' + ','.join(row)
            response.setBody(responsebody, lock=True)


class IMergeCategoriesForm(model.Schema):
    directives.widget('rename_merge_categories', SelectFieldWidget)
    rename_merge_categories = schema.List(
        title=u'Categories to Rename or Merge',
        required=True,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )
    new_category_name = schema.TextLine(
        title=u"New Category Name",
        required=True
    )


class MergeCategoriesForm(AutoExtensibleForm, form.Form):
    schema = IMergeCategoriesForm

    ignoreContext = True

    @button.buttonAndHandler(u'Rename/Merge', name='merge')
    def handle_merge(self, action):
        data, errors = self.extractData()
        if not errors:
            allcategories = api.portal.get_registry_record(reg_key)
            categories = set()
            newname = u''
            if 'form.widgets.rename_merge_categories' in self.request.form:
                if data['rename_merge_categories'] != u'':
                    categories = set(data['rename_merge_categories'])
            if 'form.widgets.new_category_name' in self.request.form:
                newname = data['new_category_name'].split(';')[0]
            if len(categories) > 0 and len(newname) > 0:
                if newname in allcategories and newname not in categories:
                    raise WidgetActionExecutionError(
                        'new_category_name',
                        Invalid(
                            u"That category name is already in use"
                        )
                    )
                    return
                for category in categories:
                    allcategories.remove(category)
                allcategories.append(newname)
                api.portal.set_registry_record(reg_key, allcategories)
                for subscriber in subscribe.all():
                    if ('categories' in subscriber and len(subscriber['categories']) > 0):
                        if len(categories.intersection(subscriber['categories'])) > 0:
                            subcat = subscriber['categories']
                            for category in categories:
                                if category in subcat:
                                    subcat.remove(category)
                            subcat.append(newname)
                            subscriber['categories'] = subcat
                self.widgets['new_category_name'].value = u''
                self.widgets['rename_merge_categories'].value = u''


class IDeleteCategoriesForm(model.Schema):
    directives.widget('delete_categories', SelectFieldWidget)
    delete_categories = schema.List(
        title=u'Categories to Delete',
        required=True,
        value_type=schema.Choice(
            vocabulary='castle.cms.vocabularies.EmailCategories'
        )
    )
    force_delete = schema.Bool(
        title=u'Force Delete',
        description=u'Delete category even if there are accounts still subscribed to it',
        default=False,
    )


class DeleteCategoriesForm(AutoExtensibleForm, form.Form):
    schema = IDeleteCategoriesForm

    ignoreContext = True

    @button.buttonAndHandler(u'Delete Categories', name='delete')
    def handle_delete(self, action):
        data, errors = self.extractData()
        if not errors:
            allcategories = api.portal.get_registry_record(reg_key)
            categories = set()
            if 'form.widgets.delete_categories' in self.request.form:
                if data['delete_categories'] != u'':
                    categories = set(data['delete_categories'])
            force_delete = 'form.widgets.force_delete' in self.request.form
            if len(categories) > 0:
                badcategories = []
                for category in categories:
                    existing_subscribers = False
                    for subscriber in subscribe.all():
                        if ('categories' in subscriber and len(subscriber['categories']) > 0):
                            if len(categories.intersection(subscriber['categories'])) > 0:
                                subcat = subscriber['categories']
                                for category in categories:
                                    if category in subcat:
                                        existing_subscribers = True
                                        if force_delete:
                                            subcat.remove(category)
                                subscriber['categories'] = subcat
                    if force_delete or not existing_subscribers:
                        allcategories.remove(category)
                    else:
                        badcategories.append(category)
                api.portal.set_registry_record(reg_key, allcategories)
                self.widgets['delete_categories'].value = ';'.join(badcategories)
                if len(badcategories) > 0:
                    raise WidgetActionExecutionError(
                        'delete_categories',
                        Invalid(
                            u'These category(s) still have subscribers. '
                            u'Select "Force Delete" if you want to unsubscribe them.'
                        )
                    )


class IAddCategoryForm(model.Schema):
    add_categories = schema.TextLine(
        title=u'New category',
        description=u'Add multiple categories at once by separating them with semicolons',
        required=True
    )


class AddCategoryForm(AutoExtensibleForm, form.Form):
    schema = IAddCategoryForm

    ignoreContext = True

    @button.buttonAndHandler(u'Add Category', name='addcat')
    def handle_addcat(self, action):
        data, errors = self.extractData()
        if not errors:
            allcategories = api.portal.get_registry_record(reg_key)
            categories = set()
            if 'add_categories' in data:
                if data['add_categories'] != u'':
                    categories = set(data['add_categories'].split(';'))
            if len(categories) > 0:
                badcategories = []
                for category in categories:
                    category = category.strip()
                    if len(category) > 0 and category not in allcategories:
                        allcategories.append(category)
                    else:
                        badcategories.append(category)
                api.portal.set_registry_record(reg_key, allcategories)
                self.widgets['add_categories'].value = ';'.join(badcategories)
                if len(badcategories) > 0:
                    raise WidgetActionExecutionError(
                        'add_categories',
                        Invalid(
                            u"That category name is already in use"
                        )
                    )


class AnnouncementsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = AnnouncementsControlPanelForm
    index = ViewPageTemplateFile('templates/announcements.pt')

    hasTexting = False

    def __init__(self, context, request):
        super(AnnouncementsControlPanel, self).__init__(context, request)
        self.email_form = SendEmailUsersForm(aq_inner(context), request)
        self.email_subscribers_form = SendEmailSubscribersForm(aq_inner(context), request)
        self.text_subscribers_form = SendTextForm(aq_inner(context), request)
        self.export_subscribers_form = ExportSubscribersForm(aq_inner(context), request)
        self.merge_categories_form = MergeCategoriesForm(aq_inner(context), request)
        self.delete_categories_form = DeleteCategoriesForm(aq_inner(context), request)
        self.add_category_form = AddCategoryForm(aq_inner(context), request)
        self.get_sub_count()

    def get_sub_count(self):
        self.categories = api.portal.get_registry_record(reg_key)
        self.sub_count = {}
        for category in self.categories:
            self.sub_count[category] = 0
        self.invalid_category = 0
        for subscriber in subscribe.all():
            if ('categories' in subscriber and len(subscriber['categories']) > 0):
                subcat = subscriber['categories']
                for category in subcat:
                    if category in self.categories:
                        self.sub_count[category] += 1
                    else:
                        self.invalid_category += 1

    def update_terms(self):
        category_vocab_terms = []
        for category in self.categories:
            category_vocab_terms.append(SimpleTerm(value=category, title=category))
        category_vocab = SimpleVocabulary(category_vocab_terms)
        self.email_subscribers_form.widgets['send_to_categories'].terms = category_vocab
        self.export_subscribers_form.widgets['export_categories'].terms = category_vocab
        self.merge_categories_form.widgets['rename_merge_categories'].terms = category_vocab
        self.delete_categories_form.widgets['delete_categories'].terms = category_vocab

    def __call__(self):
        registry = getUtility(IRegistry)
        if (registry.get('castle.plivo_auth_id') and
                registry.get('castle.plivo_auth_token') and
                registry.get('castle.plivo_phone_number')):
            self.hasTexting = True
        self.email_form.update()
        self.email_subscribers_form.update()
        self.text_subscribers_form.update()
        self.export_subscribers_form.update()
        self.merge_categories_form.update()
        self.delete_categories_form.update()
        self.add_category_form.update()
        self.update_terms()
        return super(AnnouncementsControlPanel, self).__call__()



from AccessControl.SecurityManagement import newSecurityManager
from BTrees.OOBTree import OOBTree
from castle.cms.services import twitter
from castle.cms.social import COUNT_ANNOTATION_KEY
from castle.cms.utils import index_in_es
from castle.cms.utils import retriable
from castle.cms.services.twitter import get_auth
from datetime import datetime
from plone.app.redirector.interfaces import IRedirectionStorage
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.log import logger
from tendo import singleton
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component.hooks import setSite

import json
import sys
import time
import transaction


def record_tweet_stats(site, ob, tweet):
    """
    Keep daily totals...
    Total score includes complete impression(so favorites)
    """

    site._p_jar.sync()
    annotations = IAnnotations(ob)
    social_data = annotations.get(COUNT_ANNOTATION_KEY, OOBTree())
    stats = social_data.get('twitter', {
        'total': 0
    })

    score = 1 + tweet['favorite_count']

    now = datetime.utcnow()
    if now.year not in stats:
        year = stats[now.year] = {}
    else:
        year = stats[now.year]
    if now.month not in year:
        month = year[now.month] = {}
    else:
        month = year[now.month]
    if now.day not in month:
        month[now.day] = score
    else:
        month[now.day] += score

    stats['total'] += score
    social_data['twitter'] = stats
    annotations[COUNT_ANNOTATION_KEY] = social_data
    transaction.commit()


@retriable(sync=True)
def parse_line(site, public_url, line):
    tweet = json.loads(line)
    modified = False
    for url in tweet['entities']['urls']:
        if url['expanded_url'].startswith(public_url):
            path = url['expanded_url'].replace(
                public_url, '').split('?')[0].strip('/')
            ob = None
            if path in ('', '/'):
                try:
                    ob = site[site.default_page]
                except Exception:
                    pass
            else:
                ob = site.restrictedTraverse(path, None)
            if ob is None:
                redirector = getUtility(IRedirectionStorage)
                obj_path = redirector.get(path, default=None)
                if obj_path:
                    ob = site.restrictedTraverse(obj_path, None)
            if ob:
                modified = True
                record_tweet_stats(site, ob, tweet)
    if modified:
        transaction.commit()
        index_in_es()


class StreamListener(twitter.StreamListener):
    def __init__(self, site, public_url):
        self.site = site
        self.public_url = public_url

    def on_data(self, raw_data):
        try:
            parse_line(self.site, self.public_url, raw_data)
        except Exception:
            logger.warn('error parsing tweet', exc_info=True)


def attempt_twitter_on_site(site):
    setSite(site)

    registry = getUtility(IRegistry)
    public_url = registry.get('plone.public_url', None)
    if not public_url:
        return

    auth = get_auth()

    if auth is None:
        return

    # normlize url...
    original_url = public_url = public_url.replace(
        'http://', '').replace('https://', '').strip('/')
    if public_url.count('.') > 1:
        _, _, public_url = public_url.partition('.')

    stream = twitter.Stream(auth, StreamListener(site, original_url))
    stream.filter(track=[' '.join(public_url.split('.'))])


def run(app):
    singleton.SingleInstance('twittermonitor')

    user = app.acl_users.getUser('admin')  # noqa
    newSecurityManager(None, user.__of__(app.acl_users))  # noqa

    while True:
        try:
            if 'site-id' in sys.argv:
                siteid = sys.argv['site-id']
                attempt_twitter_on_site(app[siteid])  # noqa
            else:
                for oid in app.objectIds():  # noqa
                    obj = app[oid]  # noqa
                    if IPloneSiteRoot.providedBy(obj):
                        attempt_twitter_on_site(obj)
        except KeyError:
            pass
        logger.info('Could not find valid site to monitor')
        time.sleep(10 * 60)


if __name__ == '__main__':
    run(app)  # noqa



from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME

import unittest

from castle.cms.browser import controlpanel
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING


class StatusTest(unittest.TestCase):
    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_status_control_panel_exists(self):
        view = dir(controlpanel)
        string = 'status'
        if string in view:
            self.assertTrue(True, msg='None')
        else:
            self.assertTrue(False, msg='can``t find status control panel')
